@@ -10,7 +10,7 @@ from vstools import CustomTypeError, FieldBased, FieldBasedT, core, get_render_p
 
 __all__ = [
     'sivtc',
-    'tivtc_vfr'
+    'tivtc_2pass', 'tivtc_vfr'
 ]
 
 
@@ -38,50 +38,43 @@ def sivtc(clip: vs.VideoNode, pattern: int = 0, tff: bool | FieldBasedT = True,
 
 
 main_file = os.path.realpath(sys.argv[0]) if sys.argv[0] else None
-main_file = f"{os.path.splitext(os.path.basename(str(main_file)))[0]}_"
+main_file = os.path.splitext(os.path.basename(str(main_file)))[0]
 main_file = "{yourScriptName}_" if main_file in ("__main___", "setup_") else main_file
 
 
-def tivtc_vfr(
+def tivtc_2pass(
     clip: vs.VideoNode,
-    tfm_in: Path | str = f".ivtc/{main_file}matches.txt",
-    tdec_in: Path | str = f".ivtc/{main_file}metrics.txt",
-    timecodes_out: Path | str = f".ivtc/{main_file}timecodes.txt",
-    decimate: int | bool = True,
-    tfm_args: dict[str, Any] | None = None,
-    tdecimate_args: dict[str, Any] | None = None
+    decimate_mode: int | tuple[int, int], decimate: int | bool = True,
+    tfm_in: Path | str = f".ivtc/{main_file}_matches.txt",
+    tdec_in: Path | str = f".ivtc/{main_file}_metrics.txt",
+    timecodes_out: Path | str = f".ivtc/{main_file}_timecodes.txt",
+    tfm_args: dict[str, Any] | None = None, tdecimate_args: dict[str, Any] | None = None,
+    tfm_pass_args: tuple[dict[str, Any] | None, dict[str, Any] | None] | None = None,
+    tdecimate_pass_args: tuple[dict[str, Any] | None, dict[str, Any] | None] | None = None,
 ) -> vs.VideoNode:
     """
-    Perform TFM and TDecimate on a clip that is supposed to be VFR.
+    Perform TFM and TDecimate on a clip.
 
     Includes automatic generation of a metrics/matches/timecodes txt file.
 
-    | This function took *heavy* inspiration from atomchtools.TIVTC_VFR,
-    | and is basically an improved rewrite on the concept.
-
-    .. warning::
-        | When calculating the matches and metrics for the first time, your previewer may error out!
-        | To fix this, simply refresh your previewer. If it still doesn't work, open the ``.ivtc`` directory
-        | and check if the files are **0kb**. If they are, **delete them** and run the function again.
-        | You may need to restart your previewer entirely for it to work!
-
     Dependencies:
 
     * `TIVTC <https://github.com/dubhater/vapoursynth-tivtc>`_
 
+
     :param clip:                Clip to process.
-    :param tfmIn:               File location for TFM's matches analysis.
-                                By default it will be written to ``.ivtc/{yourScriptName}_matches.txt``.
-    :param tdecIn:              File location for TDecimate's metrics analysis.
-                                By default it will be written to ``.ivtc/{yourScriptName}_metrics.txt``.
-    :param timecodes_out:       File location for TDecimate's timecodes analysis.
-                                By default it will be written to ``.ivtc/{yourScriptName}_timecodes.txt``.
-    :param decimate:            Perform TDecimate on the clip if true, else returns TFM'd clip only.
+    :param decimate_mode:       Decimate mode for both passes of TDecimate.
+    :param decimate:            Whether to perform TDecimate on the clip or returns TFM'd clip only.
                                 Set to -1 to use TDecimate without TFM.
+    :param tfm_in:              Location for TFM's matches analysis.
+    :param tdec_in:             Location for TDecimate's metrics analysis.
+    :param timecodes_out:       Location for TDecimate's timecodes analysis.
     :param tfm_args:            Additional arguments to pass to TFM.
     :param tdecimate_args:      Additional arguments to pass to TDecimate.
+    :param tfm_pass_args:       Arguments that will overwrite ``tfm_args`` in the first and second pass.
+    :param tdecimate_pass_args: Arguments that will overwrite ``tdecimate_args`` in the first and second pass.
 
-    :return:                    IVTC'd VFR clip with external timecode/matches/metrics txt files.
+    :return:                    IVTC'd clip with external timecode/matches/metrics txt files.
 
     :raises TypeError:          Invalid ``decimate`` argument is passed.
     """
@@ -95,6 +88,23 @@ def tivtc_vfr(
 
     tfm_args, tdecimate_args = tfm_args or {}, tdecimate_args or {}
 
+    if tfm_pass_args:
+        tfm_pass1, tfm_pass2 = tuple((tfm_args | (x or {})) for x in tfm_pass_args)
+    else:
+        tfm_pass1, tfm_pass2 = tfm_args.copy(), tfm_args.copy()
+
+    if tdecimate_pass_args:
+        tdecimate_pass1, tdecimate_pass2 = tuple((tdecimate_args | (x or {})) for x in tdecimate_pass_args)
+    else:
+        tdecimate_pass1, tdecimate_pass2 = tdecimate_args.copy(), tdecimate_args.copy()
+
+    if isinstance(decimate_mode, tuple):
+        tdec_mode1, tdec_mode2 = decimate_mode
+    else:
+        tdec_mode1 = tdec_mode2 = decimate_mode
+
+    tdecimate_pass1, tdecimate_pass2 = tdecimate_pass1 | dict(mode=tdec_mode1), tdecimate_pass2 | dict(mode=tdec_mode2)
+
     tfm_f = tdec_f = timecodes_f = Path()
 
     def _set_paths() -> None:
@@ -111,8 +121,8 @@ def tivtc_vfr(
             p.unlink(True)
 
     if not (tfm_f.exists() and tdec_f.exists()):
-        ivtc_clip = clip.tivtc.TFM(output=str(tfm_f), **tfm_args)
-        ivtc_clip = ivtc_clip.tivtc.TDecimate(mode=4, output=str(tdec_f), **tdecimate_args)
+        ivtc_clip = clip.tivtc.TFM(output=str(tfm_f), **tfm_pass1)
+        ivtc_clip = ivtc_clip.tivtc.TDecimate(**tdecimate_pass1, output=str(tdec_f))
 
         with get_render_progress() as pr:
             task = pr.add_task("calculating matches and metrics...", total=ivtc_clip.num_frames)
@@ -130,11 +140,50 @@ def tivtc_vfr(
         _set_paths()
 
     if decimate != -1:
-        clip = clip.tivtc.TFM(**tfm_args, input=str(tfm_f))
+        clip = clip.tivtc.TFM(**tfm_pass2, input=str(tfm_f))
 
     if decimate == 0:
         return clip
 
     return clip.tivtc.TDecimate(
-        **tdecimate_args, mode=5, hybrid=2, vfrDec=1, input=str(tdec_f), tfmIn=str(tfm_f), mkvOut=str(timecodes_f)
+        **tdecimate_pass2, input=str(tdec_f), tfmIn=str(tfm_f), mkvOut=str(timecodes_f)
+    )
+
+
+def tivtc_vfr(
+    clip: vs.VideoNode,
+    tfm_in: Path | str = f".ivtc/{main_file}_matches.txt",
+    tdec_in: Path | str = f".ivtc/{main_file}_metrics.txt",
+    timecodes_out: Path | str = f".ivtc/{main_file}_timecodes.txt",
+    decimate: int | bool = True, tfm_args: dict[str, Any] | None = None,
+    tdecimate_args: dict[str, Any] | None = None, **kwargs: Any
+) -> vs.VideoNode:
+    """
+    Perform TFM and TDecimate on a clip that is supposed to be VFR.
+
+    Includes automatic generation of a metrics/matches/timecodes txt file.
+
+    Dependencies:
+
+    * `TIVTC <https://github.com/dubhater/vapoursynth-tivtc>`_
+
+
+    :param clip:                Clip to process.
+    :param tfm_in:              Location for TFM's matches analysis.
+    :param tdec_in:             Location for TDecimate's metrics analysis.
+    :param timecodes_out:       Location for TDecimate's timecodes analysis.
+    :param decimate:            Whether to perform TDecimate on the clip or returns TFM'd clip only.
+                                Set to -1 to use TDecimate without TFM.
+    :param tfm_args:            Additional arguments to pass to TFM.
+    :param tdecimate_args:      Additional arguments to pass to TDecimate.
+    :param kwargs:              Additional kwargs for ``tivtc_2pass``.
+
+    :return:                    IVTC'd VFR clip with external timecode/matches/metrics txt files.
+
+    :raises TypeError:          Invalid ``decimate`` argument is passed.
+    """
+
+    return tivtc_2pass(
+        clip, (4, 5), decimate, tfm_in, tdec_in, timecodes_out, tfm_args,
+        tdecimate_args, None, (None, dict(hybrid=2, vfrDec=1)), **kwargs
     )
